@@ -299,12 +299,13 @@ func (r *ReconcileNetworkAddonsConfig) Reconcile(request reconcile.Request) (rec
 
 	// Everything went smooth, remove failures from NetworkAddonsConfig if there are any from
 	// previous runs.
-	r.statusManager.SetNotFailing(statusmanager.OperatorConfig)
+	r.statusManager.MarkStatusLevelNotFailing(statusmanager.OperatorConfig)
+	r.statusManager.SetFromOperator()
 
 	// From now on, r.podReconciler takes over NetworkAddonsConfig handling, it will track deployed
 	// objects if needed and set NetworkAddonsConfig.Status accordingly. However, if no pod was
 	// deployed, there is nothing that would trigger initial reconciliation. Therefore, let's
-	// perform the first check manually.
+	// perform this check manually.
 	r.statusManager.SetFromPods()
 
 	// Kubernetes sometimes fails to apply objects while we remove and recreate
@@ -488,9 +489,6 @@ func (r *ReconcileNetworkAddonsConfig) trackDeployedObjects(objs []*unstructured
 	allResources = append(allResources, deployments...)
 
 	r.podReconciler.SetResources(allResources)
-
-	// Trigger status manager to notice the change
-	r.statusManager.SetFromPods()
 }
 
 // Stop tracking current state of Deployments and DaemonSets deployed by the operator.
