@@ -11,6 +11,8 @@ import (
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
+
+	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	conditionsv1 "github.com/openshift/custom-resource-status/conditions/v1"
 	securityapi "github.com/openshift/origin/pkg/security/apis/security"
 	framework "github.com/operator-framework/operator-sdk/pkg/test"
@@ -176,10 +178,10 @@ func CheckOperatorIsReady(timeout time.Duration) {
 	By("Checking that the operator is up and running")
 	if timeout != CheckImmediately {
 		Eventually(func() error {
-			return checkForDeployment(components.Name)
+			return checkForDeployment(components.Name, false, false)
 		}, timeout, time.Second).ShouldNot(HaveOccurred(), fmt.Sprintf("Timed out waiting for the operator to become ready"))
 	} else {
-		Expect(checkForDeployment(components.Name)).ShouldNot(HaveOccurred(), "Operator is not ready")
+		Expect(checkForDeployment(components.Name, false, false)).ShouldNot(HaveOccurred(), "Operator is not ready")
 	}
 }
 
@@ -187,10 +189,10 @@ func CheckNMStateOperatorIsReady(timeout time.Duration) {
 	By("Checking that the operator is up and running")
 	if timeout != CheckImmediately {
 		Eventually(func() error {
-			return checkForGenericDeployment("nmstate-operator", "nmstate", false)
+			return checkForGenericDeployment("nmstate-operator", "nmstate", false, false)
 		}, timeout, time.Second).ShouldNot(HaveOccurred(), fmt.Sprintf("Timed out waiting for the operator to become ready"))
 	} else {
-		Expect(checkForGenericDeployment("nmstate-operator", "nmstate", false)).ShouldNot(HaveOccurred(), "Operator is not ready")
+		Expect(checkForGenericDeployment("nmstate-operator", "nmstate", false, false)).ShouldNot(HaveOccurred(), "Operator is not ready")
 	}
 }
 
@@ -245,6 +247,18 @@ func CheckForLeftoverObjects(currentVersion string) {
 	err = framework.Global.Client.List(context.Background(), &serviceAccounts, &listOptions)
 	Expect(err).NotTo(HaveOccurred())
 	Expect(serviceAccounts.Items).To(BeEmpty(), "Found leftover objects from the previous operator version")
+
+	services := corev1.ServiceList{}
+	Expect(framework.Global.Client.List(context.Background(), &services, &listOptions)).To(Succeed())
+	Expect(services.Items).To(BeEmpty(), "Found leftover objects from the previous operator version")
+
+	serviceMonitors := monitoringv1.ServiceMonitorList{}
+	Expect(framework.Global.Client.List(context.Background(), &serviceMonitors, &listOptions)).To(Succeed())
+	Expect(serviceMonitors.Items).To(BeEmpty(), "Found leftover objects from the previous operator version")
+
+	prometheusRules := monitoringv1.PrometheusRuleList{}
+	Expect(framework.Global.Client.List(context.Background(), &prometheusRules, &listOptions)).To(Succeed())
+	Expect(prometheusRules.Items).To(BeEmpty(), "Found leftover objects from the previous operator version")
 }
 
 func KeepCheckingWhile(check func(), while func()) {
@@ -296,7 +310,7 @@ func checkForComponent(component *Component) error {
 	}
 
 	for _, deployment := range component.Deployments {
-		errsAppend(checkForDeployment(deployment))
+		errsAppend(checkForDeployment(deployment, true, true))
 	}
 
 	if component.Secret != "" {
@@ -307,6 +321,18 @@ func checkForComponent(component *Component) error {
 		errsAppend(checkForMutatingWebhookConfiguration(component.MutatingWebhookConfiguration))
 	}
 
+	if component.Service != "" {
+		errsAppend(checkForService(component.Service))
+	}
+
+	if component.ServiceMonitor != "" {
+		errsAppend(checkForServiceMonitor(component.ServiceMonitor))
+	}
+
+	if component.PrometheusRule != "" {
+		errsAppend(checkForPrometheusRule(component.PrometheusRule))
+	}
+
 	return errsToErr(errs)
 }
 
@@ -338,6 +364,18 @@ func checkForComponentRemoval(component *Component) error {
 		errsAppend(checkForMutatingWebhookConfigurationRemoval(component.MutatingWebhookConfiguration))
 	}
 
+	if component.Service != "" {
+		errsAppend(checkForServiceRemoval(component.Service))
+	}
+
+	if component.ServiceMonitor != "" {
+		errsAppend(checkForServiceMonitorRemoval(component.ServiceMonitor))
+	}
+
+	if component.PrometheusRule != "" {
+		errsAppend(checkForPrometheusRuleRemoval(component.PrometheusRule))
+	}
+
 	return errsToErr(errs)
 }
 
@@ -353,26 +391,58 @@ func errsToErr(errs []error) error {
 }
 
 func checkForClusterRole(name string) error {
-	return framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &rbacv1.ClusterRole{})
+	clusterRole := rbacv1.ClusterRole{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &clusterRole)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(clusterRole.GetLabels(), "ClusterRole", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
 
 func checkForClusterRoleBinding(name string) error {
-	return framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &rbacv1.ClusterRoleBinding{})
+	clusterRoleBinding := rbacv1.ClusterRoleBinding{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &clusterRoleBinding)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(clusterRoleBinding.GetLabels(), "ClusterRoleBinding", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
 
 func checkForSecurityContextConstraints(name string) error {
-	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &securityapi.SecurityContextConstraints{})
+	scc := securityapi.SecurityContextConstraints{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &scc)
 	if isNotSupportedKind(err) {
 		return nil
 	}
-	return err
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(scc.GetLabels(), "SecurityContextConstraint", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
 
-func checkForDeployment(name string) error {
-	return checkForGenericDeployment(name, components.Namespace, true)
+func checkForDeployment(name string, checkVersionLabels, checkRelationshipLabels bool) error {
+	return checkForGenericDeployment(name, components.Namespace, checkVersionLabels, checkRelationshipLabels)
 }
 
-func checkForGenericDeployment(name, namespace string, checkLabels bool) error {
+func checkForGenericDeployment(name, namespace string, checkVersionLabels, checkRelationshipLabels bool) error {
 	deployment := appsv1.Deployment{}
 
 	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: namespace}, &deployment)
@@ -380,12 +450,21 @@ func checkForGenericDeployment(name, namespace string, checkLabels bool) error {
 		return err
 	}
 
-	if checkLabels {
+	if checkVersionLabels {
 		labels := deployment.GetLabels()
 		if labels != nil {
 			if _, operatorLabelSet := labels[cnaov1.SchemeGroupVersion.Group+"/version"]; !operatorLabelSet {
-				return fmt.Errorf("Deployment %s/%s is missing operator label", components.Namespace, name)
+				return fmt.Errorf("Deployment %s/%s is missing operator label", namespace, name)
 			}
+		} else {
+			return fmt.Errorf("Deployment %s/%s has no labels. Should have operator label", namespace, name)
+		}
+	}
+
+	if checkRelationshipLabels {
+		err := checkWorkloadRelationshipLabels([]map[string]string{deployment.GetLabels(), deployment.Spec.Template.GetLabels()}, "Deployment", name)
+		if err != nil {
+			return err
 		}
 	}
 
@@ -400,6 +479,16 @@ func checkForGenericDeployment(name, namespace string, checkLabels bool) error {
 	return nil
 }
 
+func checkWorkloadRelationshipLabels(labelMapList []map[string]string, kind, name string) error {
+	for _, labels := range labelMapList {
+		if err := checkRelationshipLabels(labels, kind, name); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 // CheckOperatorPodStability makes sure that the CNAO pod has not restarted since it started working
 func CheckOperatorPodStability(continuesDuration time.Duration) {
 	By("Checking that cnao operator pod hasn't performed any resets")
@@ -456,6 +545,11 @@ func checkForDaemonSet(name string) error {
 		}
 	}
 
+	err = checkWorkloadRelationshipLabels([]map[string]string{daemonSet.GetLabels(), daemonSet.Spec.Template.GetLabels()}, "DaemonSet", name)
+	if err != nil {
+		return err
+	}
+
 	if daemonSet.Status.NumberUnavailable > 0 || (daemonSet.Status.NumberAvailable == 0 && daemonSet.Status.DesiredNumberScheduled != 0) {
 		manifest, err := yaml.Marshal(daemonSet)
 		if err != nil {
@@ -468,11 +562,94 @@ func checkForDaemonSet(name string) error {
 }
 
 func checkForSecret(name string) error {
-	return framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &corev1.Secret{})
+	secret := corev1.Secret{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &secret)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(secret.GetLabels(), "Secret", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
 
 func checkForMutatingWebhookConfiguration(name string) error {
-	return framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &admissionregistrationv1.MutatingWebhookConfiguration{})
+	mutatingWebhookConfig := admissionregistrationv1.MutatingWebhookConfiguration{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name}, &mutatingWebhookConfig)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(mutatingWebhookConfig.GetLabels(), "MutatingWebhookConfiguration", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func checkForService(name string) error {
+	service := corev1.Service{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &service)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(service.GetLabels(), "Service", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func checkForServiceMonitor(name string) error {
+	serviceMonitor := monitoringv1.ServiceMonitor{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &serviceMonitor)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(serviceMonitor.GetLabels(), "ServiceMonitor", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func checkForPrometheusRule(name string) error {
+	prometheusRule := monitoringv1.PrometheusRule{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &prometheusRule)
+	if err != nil {
+		return err
+	}
+
+	err = checkRelationshipLabels(prometheusRule.GetLabels(), "PrometheusRule", name)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func checkRelationshipLabels(labels map[string]string, kind, name string) error {
+	expectedValues := map[string]string{
+		names.COMPONENT_LABEL_KEY:  names.COMPONENT_LABEL_DEFAULT_VALUE,
+		names.MANAGED_BY_LABEL_KEY: names.MANAGED_BY_LABEL_DEFAULT_VALUE,
+	}
+
+	for key, expectedValue := range expectedValues {
+		value, found := labels[key]
+		if !found || value != expectedValue {
+			return fmt.Errorf("%s %s is missing label %s", kind, name, key)
+		}
+	}
+
+	return nil
 }
 
 func checkForClusterRoleRemoval(name string) error {
@@ -503,6 +680,65 @@ func checkForMutatingWebhookConfigurationRemoval(name string) error {
 	return isNotFound("MutatingWebhookConfiguration", name, err)
 }
 
+func checkForServiceRemoval(name string) error {
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &corev1.Service{})
+	return isNotFound("Service", name, err)
+}
+
+func checkForServiceMonitorRemoval(name string) error {
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &monitoringv1.ServiceMonitor{})
+	return isNotFound("ServiceMonitor", name, err)
+}
+
+func checkForPrometheusRuleRemoval(name string) error {
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: name, Namespace: components.Namespace}, &monitoringv1.PrometheusRule{})
+	return isNotFound("PrometheusRule", name, err)
+}
+
+func getMonitoringEndpoint() (*corev1.Endpoints, error) {
+	By("Finding CNAO prometheus endpoint")
+	endpoint := &corev1.Endpoints{}
+	err := framework.Global.Client.Get(context.Background(), types.NamespacedName{Name: MonitoringComponent.Service, Namespace: components.Namespace}, endpoint)
+	if err != nil {
+		return nil, err
+	}
+	return endpoint, nil
+}
+
+func scrapeEndpointAddress(epAddress corev1.EndpointAddress, epPort int32) (string, error) {
+	stdout, err := Kubectl("exec", "-n", epAddress.TargetRef.Namespace, epAddress.TargetRef.Name, "--", "curl", "-L", "-s", "-k", fmt.Sprintf("http://%s:%d/metrics", epAddress.IP, epPort))
+	if err != nil {
+		return "", err
+	}
+	return stdout, nil
+}
+
+func GetScrapedDataFromMonitoringEndpoint() (string, error) {
+	endpoint, err := getMonitoringEndpoint()
+	if err != nil {
+		return "", err
+	}
+
+	By("scraping the metrics endpoint on CNAO pod")
+	epPort := endpoint.Subsets[0].Ports[0].Port
+	for _, epAddress := range endpoint.Subsets[0].Addresses {
+		if !strings.HasPrefix(epAddress.TargetRef.Name, components.Name) {
+			continue
+		}
+		return scrapeEndpointAddress(epAddress, epPort)
+	}
+	return "", errors.New("no endpoint target ref name matches CNAO component")
+}
+
+func FindMetric(metrics string, expectedMetric string) string {
+	for _, line := range strings.Split(metrics, "\n") {
+		if strings.HasPrefix(line, expectedMetric+" ") {
+			return line
+		}
+	}
+	return ""
+}
+
 func isNotFound(componentType string, componentName string, clientGetOutput error) error {
 	if clientGetOutput != nil {
 		if apierrors.IsNotFound(clientGetOutput) {
